/* Fichier "gtlglw_cb.c"
 * callbacks de la fenetre OpenGL de l'interface GTK+-2.x
 * 
 * Ce fichier a t crit de telle sorte qu'il ne fasse appel
 *  aucun code de rendu OpenGL _directement_ pour des raisons de 
 * portabilit vers d'autres interfaces.
 * Il ne faut appeler ici que des fonctions de gl_generic.c
 */

#include "projet.h"

int intro_running = 1;

/* ======================================================
 * on_gl_window_destroy
 *
 * Description: appele quand le signal "destroy" est recu
 *  sur la fenetre GtkGLExt.
 * =====================================================
 */
void
on_gl_window_destroy (GtkObject * object, gpointer user_data)
{
  gl_window_idle_remove (GTK_WIDGET (user_data));
  gtk_widget_destroy (GTK_WIDGET (user_data));

  gtk_main_quit ();
}

/* ======================================================
 * on_gl_window_draw
 *
 * Description: appele quand le signal "expose_event" est recu
 *  sur la fenetre GtkGLExt.
 * =====================================================
 */
gboolean
on_gl_window_draw (GtkWidget * widget, GdkEventExpose * event, gpointer data)
{
  GdkGLContext *glcontext = gtk_widget_get_gl_context (widget);
  GdkGLDrawable *gldrawable = gtk_widget_get_gl_drawable (widget);

  /*** OpenGL BEGIN ***/
  if (!gdk_gl_drawable_gl_begin (gldrawable, glcontext))
    return FALSE;

  if (!intro_running)
    gl_generic_render_scene ();
  
  else glintro_draw_2D_intro ();

  if (gdk_gl_drawable_is_double_buffered (gldrawable))
    gdk_gl_drawable_swap_buffers (gldrawable);
  else
    glFlush ();

  gdk_gl_drawable_gl_end (gldrawable);
  /*** OpenGL END ***/

  return TRUE;
}

/* ======================================================
 * on_gl_window_reshape
 *
 * Description: appele quand le signal "configure_event" est recu
 *  sur la fenetre GtkGLExt, i.e. quand on redimensionne,
 *  ou quand la fenetre apparait apres avoir t cache
 * =====================================================
 */
gboolean
on_gl_window_reshape (GtkWidget * widget,
		      GdkEventConfigure * event, gpointer data)
{
  GdkGLContext *glcontext = gtk_widget_get_gl_context (widget);
  GdkGLDrawable *gldrawable = gtk_widget_get_gl_drawable (widget);

  /*** OpenGL BEGIN ***/
  if (!gdk_gl_drawable_gl_begin (gldrawable, glcontext))
    return FALSE;

  if (!intro_running)
    gl_generic_resize (widget->allocation.width, widget->allocation.height);

  gdk_gl_drawable_gl_end (gldrawable);
  /*** OpenGL END ***/

  return TRUE;
}


/* ======================================================
 * on_gl_window_init
 *
 * Description: appele quand le signal "realize" est recu
 *  sur la fenetre GtkGLExt, i.e. quand la fenetre est cree.
 * =====================================================
 */
void
on_gl_window_init (GtkWidget * widget, gpointer data)
{
  GdkGLContext *glcontext = gtk_widget_get_gl_context (widget);
  GdkGLDrawable *gldrawable = gtk_widget_get_gl_drawable (widget);

  /*** OpenGL BEGIN ***/
  if (!gdk_gl_drawable_gl_begin (gldrawable, glcontext))
    return;

  gl_generic_init_2D_intro ();

  gdk_gl_drawable_gl_end (gldrawable);
  /*** OpenGL END ***/
}

/* ======================================================
 * on_gl_window_idle
 *
 * Description: appele quand on n'a rien d'autre  faire.
 * =====================================================
 */
gboolean
on_gl_window_idle (GtkWidget * widget)
{
  gtk_widget_queue_draw (widget);

  return TRUE;
}

static guint idle_id = 0;

/* ======================================================
 * gl_window_idle_add
 *
 * Description:
 * =====================================================
 */
void
gl_window_idle_add (GtkWidget * widget)
{
  if (idle_id == 0) {
    idle_id = gtk_idle_add_priority (GDK_PRIORITY_REDRAW,
				     (GtkFunction) on_gl_window_idle, widget);
  }
}

/* ======================================================
 * gl_window_idle_remove
 *
 * Description:
 * =====================================================
 */
void
gl_window_idle_remove (GtkWidget * widget)
{
  if (idle_id != 0) {
    gtk_idle_remove (idle_id);
    idle_id = 0;
  }
}

/* ======================================================
 * on_gl_window_map
 *
 * Description:
 * =====================================================
 */
gboolean
on_gl_window_map (GtkWidget * widget, GdkEventAny * event, gpointer data)
{
  gl_window_idle_add (widget);

  return TRUE;
}

/* ======================================================
 * on_gl_window_unmap
 *
 * Description:
 * =====================================================
 */
gboolean
on_gl_window_unmap (GtkWidget * widget, GdkEventAny * event, gpointer data)
{
  gl_window_idle_remove (widget);

  return TRUE;
}

/* ======================================================
 * on_gl_window_visible
 *
 * Description: appele quand le signal "visibility_notify_event" est recu
 *  sur la fenetre GtkGLExt, i.e. quand il y a un changement
 *  de visibilit de la fenetre (partiellement recouverte etc...)
 * =====================================================
 */
gboolean
on_gl_window_visible (GtkWidget * widget,
		      GdkEventVisibility * event, gpointer data)
{
  if (event->state == GDK_VISIBILITY_FULLY_OBSCURED)
    gl_window_idle_remove (widget);
  else
    gl_window_idle_add (widget);

  return TRUE;
}

/* ======================================================
 * on_gl_window_key
 *
 * Description: appele quand le signal "key_press_event" est recu
 *  sur la fenetre GtkGLExt, i.e. quand on appuie sur une touche
 * =====================================================
 */
gboolean
on_gl_window_key (GtkWidget * widget, GdkEventKey * event, gpointer data)
{
  if (cam_get_control_mode () == CAMERA_CONTROL_TERRAIN) {
    switch (event->keyval) {
    case GDK_Up:
      cam_move_cam_forwbackw (2);
      break;
    case GDK_Down:
      cam_move_cam_forwbackw (-2);
      break;
    case GDK_Left:
      cam_rotate_terrain_leftright (0.1);
      break;
    case GDK_Right:
      cam_rotate_terrain_leftright (-0.1);
      break;
    case GDK_Page_Up:
      cam_rotate_cam_updown (-0.03);
      break;
    case GDK_Page_Down:
      cam_rotate_cam_updown(0.03);
      break;
    case GDK_Escape:
      cam_set_default_view ();
      break;
    case GDK_w:
      gl_generic_switch_wireframe ();
      break;
    default:
      return FALSE;
    }
  }

  else {
    switch (event->keyval) {
    case GDK_Up:
      cam_move_cam_forwbackw (2);
      break;
    case GDK_Down:
      cam_move_cam_forwbackw (-2);
      break;
    case GDK_Left:
      cam_rotate_cam_leftright (0.05);
      break;
    case GDK_Right:
      cam_rotate_cam_leftright (-0.05); 
      break;
    case GDK_q:
      cam_strafe_cam_leftright (-2);
      break;
    case GDK_d:
      cam_strafe_cam_leftright (2);
      break;
    case GDK_z:
      cam_move_cam_updown (2);
      break;
    case GDK_s:
      cam_move_cam_updown (-2);
      break;
    case GDK_Page_Up:
      cam_rotate_cam_updown (-0.03);
      break;
    case GDK_Page_Down:
      cam_rotate_cam_updown(0.03);
      break;
    case GDK_Escape:
      cam_set_default_view ();
      break;
    case GDK_w:
      gl_generic_switch_wireframe ();
      break;
    default:
      return FALSE;
    }
  }

  gtk_widget_queue_draw (widget);

  return TRUE;
}
