#include "projet.h"

static gchar *filename = NULL;
static gint already_saved = 0;
extern GtkWidget *gl_window;

extern TER_param_hyperb_t hyp_param;
extern TER_param_middisp_t middisp_param;
extern TER_param_diamsquare_t diamsq_param;
extern TER_param_fault_t fault_param;
extern TER_param_particle_t particle_param;

int newalgow_opened = 0;
int filtersw_opened = 0;

extern int intro_running;

void
gtkglw_activate_buttons ()
{
  GtkWidget *widg;

  /* active les boutons Refresh et Save */
  widg = lookup_widget (GTK_WIDGET (gl_window), "renderw_saveb");
  gtk_widget_set_sensitive (widg, TRUE);
  widg = lookup_widget (GTK_WIDGET (gl_window), "renderw_refrb");
  gtk_widget_set_sensitive (widg, TRUE);
  widg = lookup_widget (GTK_WIDGET (gl_window), "renderw_filtersb");
  gtk_widget_set_sensitive (widg, TRUE);
  widg = lookup_widget (GTK_WIDGET (gl_window), "enregistrer1");
  gtk_widget_set_sensitive (widg, TRUE);
  widg = lookup_widget (GTK_WIDGET (gl_window), "enregistrer_sous1");
  gtk_widget_set_sensitive (widg, TRUE);
  widg = lookup_widget (GTK_WIDGET (gl_window), "exporter1");
  gtk_widget_set_sensitive (widg, TRUE);
  widg = lookup_widget (GTK_WIDGET (gl_window), "filtres1");
  gtk_widget_set_sensitive (widg, TRUE);
  
  if (intro_running) { 
    gl_generic_init ();
    intro_running=0;
  }
}

/* ======================================================
 * on_nouveau1_activate
 *
 * Description: 
 * =====================================================
 */
void
on_nouveau1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  GtkWidget *newalgow;

  if (!newalgow_opened) {
    newalgow = create_cal_window ();
    gtk_widget_show (newalgow);
    newalgow_opened = 1;
  }
}

void
on_quitter1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  gl_window_idle_remove (GTK_WIDGET (user_data));
  gtk_widget_destroy (GTK_WIDGET (user_data));

  gtk_main_quit ();
}

void
callback_ouvrir_ok (GtkWidget * w, GtkFileSelection * fs)
{
  gchar *filename_open = NULL;
  void *params = NULL;
  int algo;

  filename_open =
    g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (w)));

  algo = TER_read_algo (filename_open);
  if (algo < 0) {
    fprintf (stderr, "An error occured while opening terrain\n");
    /* TODO: ouvrir une boite de dialogue */
    return;
  }

  switch (algo) {
  case TERR_FLAT:
    break;
  case TERR_MIDDISP:
    params = &middisp_param;
    break;
  case TERR_SPHERE:
    params = &hyp_param;
    break;
  case TERR_HYPERB:
    params = &hyp_param;
    break;
  case TERR_DIAM:
    params = &diamsq_param;
    break;
  case TERR_FAULT:
    params = &fault_param;
    break;
  case TERR_PARTICLE:
    params = &particle_param;
    break;

  default:
    fprintf (stderr, "Warning: unknown algorithm\n");
    return;
    break;
  }

  if (TER_import_terrain (terrain, params, filename_open) == 0) {
    fprintf (stderr, "An error occured while opening terrain\n");
    /* TODO: ouvrir une boite de dialogue */
    return;
  }

  if (terrain->heightmap)
    tessallate_terrain (terrain);

  else {
    fprintf (stderr, "Importation failed\n");
    return;
  }

  gtkglw_activate_buttons ();
  if (intro_running) { 
    gl_generic_init ();
    intro_running=0;
  }

  g_free (filename_open);
  gtk_widget_destroy (GTK_WIDGET (w));
}

void
on_ouvrir1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  GtkWidget *filew;
  filew = gtk_file_selection_new (_("Ouvrir..."));

  gtk_signal_connect_object (GTK_OBJECT
			     (GTK_FILE_SELECTION (filew)->ok_button),
			     "clicked", (GtkSignalFunc) callback_ouvrir_ok,
			     filew);

  gtk_signal_connect_object (GTK_OBJECT
			     (GTK_FILE_SELECTION (filew)->cancel_button),
			     "clicked", (GtkSignalFunc) gtk_widget_destroy,
			     GTK_OBJECT (filew));

  gtk_signal_connect (GTK_OBJECT (filew), "destroy",
		      (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT (filew));

  gtk_widget_show (filew);
}

void
callback_enregsous_ok (GtkWidget * w, GtkFileSelection * fs)
{
  /*g_print ("sauvegarde de %s\n", (gchar *)
     gtk_file_selection_get_filename (GTK_FILE_SELECTION (w))); */

  if (terrain->generated) {
    if (filename)
      g_free (filename);
    filename = g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION
							  (w)));
    /*terr_export_png_bw (terrain, filename); */
    if (TER_export_terrain (terrain, filename) == 0) {
      fprintf (stderr, "An error occured while saving terrain\n");
      /* TODO: ouvrir une boite de dialogue */
    }
    already_saved = 1;
  }

  gtk_widget_destroy (GTK_WIDGET (w));
}

void
on_enregistrer_sous1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  GtkWidget *filew;
  filew = gtk_file_selection_new (_("Enregistrer sous..."));
  /*gtk_file_selection_set_filename (GTK_FILE_SELECTION (filew),
     "heightmap.png"); */
  gtk_file_selection_set_filename (GTK_FILE_SELECTION (filew), "terrain.ter");

  gtk_signal_connect_object (GTK_OBJECT
			     (GTK_FILE_SELECTION (filew)->ok_button),
			     "clicked", (GtkSignalFunc) callback_enregsous_ok,
			     filew);

  gtk_signal_connect_object (GTK_OBJECT
			     (GTK_FILE_SELECTION (filew)->cancel_button),
			     "clicked", (GtkSignalFunc) gtk_widget_destroy,
			     GTK_OBJECT (filew));

  gtk_signal_connect (GTK_OBJECT (filew), "destroy",
		      (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT (filew));

  gtk_widget_show (filew);
}

void
on_enregistrer1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  if (already_saved) {
    if (TER_export_terrain (terrain, filename) == 0) {
      fprintf (stderr, "An error occured while saving terrain\n");
      /* TODO: ouvrir une boite de dialogue */
    }
  }

  else
    on_enregistrer_sous1_activate (NULL, NULL);
}

void
callback_exporter_ok (GtkWidget * w, GtkFileSelection * fs)
{
  gchar *filename_export = NULL;

  if (terrain->generated) {

    filename_export =
      g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (w)));
    if (TER_export_png_bw (terrain, filename_export) == 0) {
      fprintf (stderr, "An error occured while exporting terrain\n");
      /* TODO: ouvrir une boite de dialogue */
    }
  }

  gtk_widget_destroy (GTK_WIDGET (w));
}

void
on_exporter1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  GtkWidget *filew;
  filew = gtk_file_selection_new (_("Exporter en PNG..."));
  gtk_file_selection_set_filename (GTK_FILE_SELECTION (filew),
				   "heightmap.png");

  gtk_signal_connect_object (GTK_OBJECT
			     (GTK_FILE_SELECTION (filew)->ok_button),
			     "clicked", (GtkSignalFunc) callback_exporter_ok,
			     filew);

  gtk_signal_connect_object (GTK_OBJECT
			     (GTK_FILE_SELECTION (filew)->cancel_button),
			     "clicked", (GtkSignalFunc) gtk_widget_destroy,
			     GTK_OBJECT (filew));

  gtk_signal_connect (GTK_OBJECT (filew), "destroy",
		      (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT (filew));

  gtk_widget_show (filew);
}

void
on_renderw_openb_clicked (GtkObject * object, gpointer user_data)
{
  on_ouvrir1_activate (NULL, NULL);
}

void
on_renderw_saveb_clicked (GtkObject * object, gpointer user_data)
{
  on_enregistrer1_activate (NULL, NULL);
}

void
on_renderw_newb_clicked (GtkObject * object, gpointer user_data)
{
  on_nouveau1_activate (NULL, NULL);
}


void
on_renderw_refrb_clicked (GtkObject * object, gpointer user_data)
{
  if (terrain->spec_param) {
    ui_generic_generate_new_terrain (terrain);
  }
  
  else
    fprintf (stderr, "Error: no parameter found\n");
}

void
on_about1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  GtkWidget *aboutd = create_about_dialog ();

  gtk_widget_show (aboutd);
}

void
on_filtres1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  GtkWidget *filterw;

  if (!filtersw_opened) {
    filterw = create_filters_window ();
    gtk_widget_show (filterw);
    filtersw_opened = 1;
  }
}


void
on_renderw_filtersb_clicked (GtkObject * object, gpointer user_data)
{
  on_filtres1_activate (NULL, NULL);
}

void
on_backfaceculling1_activate (GtkCheckMenuItem * menuitem, gpointer user_data)
{
  gl_generic_switch_bfculling ();
}

void
on_wireframe1_activate (GtkCheckMenuItem * menuitem, gpointer user_data)
{
  gl_generic_switch_wireframe ();
}

void
on_terrain1_activate (GtkRadioMenuItem * menuitem, gpointer user_data)
{
  cam_set_control_mode (CAMERA_CONTROL_TERRAIN);
}

void
on_camera1_activate (GtkRadioMenuItem * menuitem, gpointer user_data)
{
  cam_set_control_mode (CAMERA_CONTROL_CAMERA);
}
